package me.ChrisvA.MinecartExtra;


import java.util.HashMap;
import java.util.List;
import java.util.UUID;
import java.util.logging.Logger;

import org.bukkit.ChatColor;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Entity;
import org.bukkit.entity.Item;
import org.bukkit.entity.Player;
import org.bukkit.entity.StorageMinecart;
import org.bukkit.inventory.ItemStack;
import org.bukkit.plugin.java.JavaPlugin;

public class MinecartExtra extends JavaPlugin {
  
	private Logger log = Logger.getLogger("Minecraft");
	
	private MinecartExtraVehicleListener vehicleListener; 
	
	private HashMap<UUID, MinecartCollector> collectionMinecarts;
	
	// Hashmap of players that type collect create
	//            PlayerId, Ticks since creation
	private HashMap<UUID, Integer> playersCollectCreate;
	
	private Boolean clean = false;
	
	private MinecartExtraConfig config; //PluginConfig
	private MinecartExtraEconomy economy; //Economy API
	private MinecartExtraPermissions perm; //Permissions
	private MinecartExtraString plstr; //PluginStrings
	
	public final String cmdCollect = "collect";
	
	public void onEnable(){
		
		// List of collecting minecarts
        collectionMinecarts = new HashMap<UUID, MinecartCollector> ();
        
        // players that type collect create
        playersCollectCreate = new HashMap<UUID, Integer> ();
        
        vehicleListener = new MinecartExtraVehicleListener(this);
        
        config = new MinecartExtraConfig(this);
        perm = new MinecartExtraPermissions(this);
        plstr = new MinecartExtraString(this, config.getLang());
        getServer().getPluginManager().registerEvents(vehicleListener, this);
        if(config.isUseEconomy()){
        	economy = new MinecartExtraEconomy(this);
        }else{
        	economy = null;
        }
        
        
        //tickszaehler aktivieren
        getServer().getScheduler().scheduleAsyncRepeatingTask(this, new Runnable() {

            public void run() {
                cleanup();
            }
        }, config.getCollectCleanup(), config.getCollectCleanup());
        
        getServer().getScheduler().scheduleAsyncRepeatingTask(this, new Runnable() {

            public void run() {
                checkTimeOut();
            }
        }, (config.getCollectCheckTimeout() + 7), config.getCollectCheckTimeout());
        //increase initial dealy to prevent colliding with cleanup
        
        log.info(getDescription().getName() + " loaded with that config:");
        sendInfo(getServer().getConsoleSender());
    }
	
    public void onDisable(){

    }

    public void StorageMinecartMove (StorageMinecart cart){
    	if (clean == true){
    		//Prevent access error
    		//getServer().broadcastMessage(ChatColor.AQUA + "Move verworfen");
    		return;
    	}
		if (collectionMinecarts.containsKey(cart.getUniqueId())){
			if (collectionMinecarts.get(cart.getUniqueId()).CollectionTime()){
				//Collect items
				
				List<Entity> near = cart.getNearbyEntities(config.getCollectHradius(), 
						config.getCollectVradius(), config.getCollectHradius());
				
				//getServer().broadcastMessage("Collect" + near.size());
				for (Entity e: near){
					
					if (e instanceof Item){
						Item i = (Item) e;
						if(!i.isDead()){
							//Item Einpacken und entfernen
							HashMap<Integer, ItemStack> rest = cart.getInventory().addItem(i.getItemStack());
							//getServer().broadcastMessage(ChatColor.AQUA + "NichtVerwendbar" + rest.size());
							if(rest.size() == 0){
								i.remove();
							}else{
								for ( int key : rest.keySet() ){
									i.getItemStack().setAmount(rest.get(key).getAmount());
								}
							}
							
						}
					}
				}
			}
		}else if(config.isCollectAutodetect()){
			collectionMinecarts.put(cart.getUniqueId(), 
					new MinecartCollector(cart.getLocation().getWorld(), config.getCollectRate()));
			
			//getServer().broadcastMessage("Insert" + collectionMinecarts.size());
		}
    }
    
    private void checkTimeOut(){
    	if(clean == true){
    		return;
    	}
    	clean = true;
    	for (UUID key : collectionMinecarts.keySet()){
    		if ( collectionMinecarts.get(key).isPlayerCart()){
    			// Check if time is out for Minecarts added by players
    			if((collectionMinecarts.get(key).getLifeTime()) > config.getCollectTimeCollection() ){
	    			//getServer().broadcastMessage(ChatColor.AQUA + "Premove");
	    			collectionMinecarts.remove(key);
	    		}
    		}
    	}
    	
    	for (UUID id : playersCollectCreate.keySet()){
    		if (playersCollectCreate.get(id) >= config.getCollectTimeplayers() ){
    			playersCollectCreate.remove(id);
    		}else{
    			Integer num = playersCollectCreate.get(id);
    			num += this.config.getCollectCheckTimeout();
    			playersCollectCreate.put(id, num);
    		}
    	}
    	
    	clean = false;
    }
    
    private void cleanup(){
    	if(clean == true){
    		return;
    	}
    	clean = true;
    	//getServer().broadcastMessage(ChatColor.AQUA + "cleanup");
    	for (UUID key : collectionMinecarts.keySet()){
    		if ( !collectionMinecarts.get(key).isPlayerCart()){
    			if((collectionMinecarts.get(key).getNoMoveTime()) > config.getCollectCleanup()){
	    			//The Cart doesn't move since last cleanup
	    			//getServer().broadcastMessage(ChatColor.AQUA + "remove");
	    			collectionMinecarts.remove(key);
	    		}
    		}
    	}
    	clean = false;
    }
    
    public boolean onCommand(CommandSender sender, Command cmd, String commandLabel, String[] args){
    	if(cmd.getName().equalsIgnoreCase(cmdCollect)){
    		
    		if (args.length == 0){
    			return false;
    		}
    		if (!(sender instanceof Player)){
    			//By Console or RemoteConsole
    			if (args[0].equalsIgnoreCase("info")){
    				sendInfo (sender);
    				return true;
    			}
    		}else{
    			//By Player
    			Player p = (Player) sender;
    			if (args[0].equalsIgnoreCase("info")){
    				
    				if (perm.hasInfo(p)){
    					sendInfo (sender);
        			}else{
        				sendNoPermissions(p);
        			}
    				return true;
    			}else if(args[0].equalsIgnoreCase("create")){
    				if (perm.hasCreateAutostrage(p)){
   						playersCollectCreate.put(p.getUniqueId(), 0);
   						p.sendMessage(ChatColor.YELLOW + plstr.getLeftKlickAMinecart());
    				}else{
    					sendNoPermissions(p);
    				}
    				return true;
    			}
    			//unknown argument
    			return false; 
    		}
    		
    		return true;
    	}
    	
    	//unknown command
    	return false;
    }


    private void sendNoPermissions(Player p) {
		p.sendMessage(ChatColor.RED + plstr.getNoPermissions());
	}

	private void sendInfo (CommandSender sender){
    	sender.sendMessage(getDescription().getFullName());
    	sender.sendMessage(plstr.getCollectMinecarts() + ": " + collectionMinecarts.size());
    	sender.sendMessage("hradius: " + config.getCollectHradius());
    	sender.sendMessage("vradius: " + config.getCollectVradius());
    	sender.sendMessage("rate: " + config.getCollectRate());
    	sender.sendMessage("cleanup: " + config.getCollectCleanup());
    	sender.sendMessage("CheckTimeOut: " + config.getCollectCheckTimeout());
    	sender.sendMessage("timeplayers: " + config.getCollectTimeplayers());
    	sender.sendMessage("timecollection: " + config.getCollectTimeCollection());
    	sender.sendMessage("autodetect: " + config.isCollectAutodetect());
    	sender.sendMessage("use economy: " + config.isUseEconomy());
    	sender.sendMessage("price CollectCreate: " + config.getPriceCollectCreate());
    	sender.sendMessage("lang: " + config.getLang());
    	sender.sendMessage(plstr.getLanguage());
    }

	 public boolean StorageMinecartDamage (StorageMinecart cart, Player player){
		 //Check if the Player has typed the Collec create Command
		 
		 if (!playersCollectCreate.containsKey(player.getUniqueId())){
			 return false;
		 }else{
			 playersCollectCreate.remove(player.getUniqueId());
		 }
		 
		 if (!collectionMinecarts.containsKey(cart.getUniqueId())){
			 //check for economy usage 
			 if(!this.config.isUseEconomy() || economy.take(player, this.config.getPriceCollectCreate())){
				// register Minecart as a minecart added by a player
				 collectionMinecarts.put(cart.getUniqueId(),
						 new MinecartCollector(cart.getLocation().getWorld(), config.getCollectRate(), true));
				 player.sendMessage(ChatColor.YELLOW + plstr.getMinecartAddSucc()); 
			 }
		 }else{
			player.sendMessage(ChatColor.RED + plstr.getMincartAlreadyAdded());
		 }
		 // for cancelling event
		 return true;
	 }
	 
	 public MinecartExtraString getStrings(){
		 return this.plstr;
	 }

}
